@@ -138,7 +138,7 @@ static void *sigrsqrt_new(void)
 static t_int *sigrsqrt_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = *(t_int *)(w+3);
+    int n = (int)w[3];
     while (n--)
     {
         t_sample f = *in++;
@@ -197,7 +197,7 @@ static void *sigsqrt_new(void)
 t_int *sigsqrt_perform(t_int *w)    /* not static; also used in d_fft.c */
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = *(t_int *)(w+3);
+    int n = (int)w[3];
     while (n--)
     {
         t_sample f = *in++;
@@ -253,7 +253,7 @@ static void *sigwrap_new(void)
 static t_int *sigwrap_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = (t_int)w[3];
+    int n = (int)w[3];
     while (n--)
     {
         t_sample f = *in++;
@@ -268,7 +268,7 @@ static t_int *sigwrap_perform(t_int *w)
 static t_int *sigwrap_old_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = (t_int)w[3];
+    int n = (int)w[3];
     while (n--)
     {
         t_sample f = *in++;
@@ -316,7 +316,7 @@ static void *mtof_tilde_new(void)
 static t_int *mtof_tilde_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = (t_int)w[3];
+    int n = (int)w[3];
     for (; n--; in++, out++)
     {
         t_sample f = *in;
@@ -365,7 +365,7 @@ static void *ftom_tilde_new(void)
 static t_int *ftom_tilde_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = (t_int)w[3];
+    int n = (int)w[3];
     for (; n--; in++, out++)
     {
         t_sample f = *in;
@@ -409,7 +409,7 @@ static void *dbtorms_tilde_new(void)
 static t_int *dbtorms_tilde_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = (t_int)w[3];
+    int n = (int)w[3];
     for (; n--; in++, out++)
     {
         t_sample f = *in;
@@ -459,7 +459,7 @@ static void *rmstodb_tilde_new(void)
 static t_int *rmstodb_tilde_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = (t_int)w[3];
+    int n = (int)w[3];
     for (; n--; in++, out++)
     {
         t_sample f = *in;
@@ -508,7 +508,7 @@ static void *dbtopow_tilde_new(void)
 static t_int *dbtopow_tilde_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = (t_int)w[3];
+    int n = (int)w[3];
     for (; n--; in++, out++)
     {
         t_sample f = *in;
@@ -558,7 +558,7 @@ static void *powtodb_tilde_new(void)
 static t_int *powtodb_tilde_perform(t_int *w)
 {
     t_sample *in = (t_sample *)w[1], *out = (t_sample *)w[2];
-    t_int n = (t_int)w[3];
+    int n = (int)w[3];
     for (; n--; in++, out++)
     {
         t_sample f = *in;
@@ -645,7 +645,7 @@ typedef struct _exp_tilde
     t_float x_f;
 } t_exp_tilde;
 
-static void *exp_tilde_new( void)
+static void *exp_tilde_new(void)
 {
     t_exp_tilde *x = (t_exp_tilde *)pd_new(exp_tilde_class);
     outlet_new(&x->x_obj, &s_signal);
@@ -739,7 +739,7 @@ typedef struct _abs_tilde
     t_float x_f;
 } t_abs_tilde;
 
-static void *abs_tilde_new( void)
+static void *abs_tilde_new(void)
 {
     t_abs_tilde *x = (t_abs_tilde *)pd_new(abs_tilde_class);
     outlet_new(&x->x_obj, &s_signal);
