@@ -513,7 +513,7 @@ t_float glist_pixelstox(t_glist *x, t_float xpix)
         coordinates (x1, etc.) specifies the coordinate range
         of a one-pixel square at top left of the window. */
     if (!x->gl_isgraph)
-        return (x->gl_x1 + (x->gl_x2 - x->gl_x1) * xpix);
+        return (x->gl_x1 + (x->gl_x2 - x->gl_x1) * xpix / x->gl_zoom);
 
         /* if we're a graph when shown on parent, but own our own
         window right now, our range in our coordinates (x1, etc.) is spread
@@ -538,7 +538,7 @@ t_float glist_pixelstox(t_glist *x, t_float xpix)
 t_float glist_pixelstoy(t_glist *x, t_float ypix)
 {
     if (!x->gl_isgraph)
-        return (x->gl_y1 + (x->gl_y2 - x->gl_y1) * ypix);
+        return (x->gl_y1 + (x->gl_y2 - x->gl_y1) * ypix / x->gl_zoom);
     else if (x->gl_isgraph && x->gl_havewindow)
         return (x->gl_y1 + (x->gl_y2 - x->gl_y1) *
                 (ypix) / (x->gl_screeny2 - x->gl_screeny1));
@@ -557,7 +557,7 @@ t_float glist_pixelstoy(t_glist *x, t_float ypix)
 t_float glist_xtopixels(t_glist *x, t_float xval)
 {
     if (!x->gl_isgraph)
-        return ((xval - x->gl_x1) / (x->gl_x2 - x->gl_x1));
+        return (((xval - x->gl_x1) * x->gl_zoom) / (x->gl_x2 - x->gl_x1));
     else if (x->gl_isgraph && x->gl_havewindow)
         return (x->gl_screenx2 - x->gl_screenx1) *
             (xval - x->gl_x1) / (x->gl_x2 - x->gl_x1);
@@ -574,7 +574,7 @@ t_float glist_xtopixels(t_glist *x, t_float xval)
 t_float glist_ytopixels(t_glist *x, t_float yval)
 {
     if (!x->gl_isgraph)
-        return ((yval - x->gl_y1) / (x->gl_y2 - x->gl_y1));
+        return (((yval - x->gl_y1) * x->gl_zoom) / (x->gl_y2 - x->gl_y1));
     else if (x->gl_isgraph && x->gl_havewindow)
         return (x->gl_screeny2 - x->gl_screeny1) *
                 (yval - x->gl_y1) / (x->gl_y2 - x->gl_y1);
@@ -610,10 +610,10 @@ t_float glist_dpixtody(t_glist *x, t_float dypix)
 int text_xpix(t_text *x, t_glist *glist)
 {
     if (glist->gl_havewindow || !glist->gl_isgraph)
-        return (x->te_xpix);
+        return (x->te_xpix * glist->gl_zoom);
     else if (glist->gl_goprect)
         return (glist_xtopixels(glist, glist->gl_x1) +
-            x->te_xpix - glist->gl_xmargin);
+            glist->gl_zoom * (x->te_xpix - glist->gl_xmargin));
     else return (glist_xtopixels(glist,
             glist->gl_x1 + (glist->gl_x2 - glist->gl_x1) *
                 x->te_xpix / (glist->gl_screenx2 - glist->gl_screenx1)));
@@ -622,10 +622,10 @@ int text_xpix(t_text *x, t_glist *glist)
 int text_ypix(t_text *x, t_glist *glist)
 {
     if (glist->gl_havewindow || !glist->gl_isgraph)
-        return (x->te_ypix);
+        return (x->te_ypix * glist->gl_zoom);
     else if (glist->gl_goprect)
         return (glist_ytopixels(glist, glist->gl_y1) +
-            x->te_ypix - glist->gl_ymargin);
+            glist->gl_zoom * (x->te_ypix - glist->gl_ymargin));
     else return (glist_ytopixels(glist,
             glist->gl_y1 + (glist->gl_y2 - glist->gl_y1) *
                 x->te_ypix / (glist->gl_screeny2 - glist->gl_screeny1)));
@@ -869,8 +869,8 @@ static void graph_graphrect(t_gobj *z, t_glist *glist,
     int x1 = text_xpix(&x->gl_obj, glist);
     int y1 = text_ypix(&x->gl_obj, glist);
     int x2, y2;
-    x2 = x1 + x->gl_pixwidth;
-    y2 = y1 + x->gl_pixheight;
+    x2 = x1 + x->gl_zoom * x->gl_pixwidth;
+    y2 = y1 + x->gl_zoom * x->gl_pixheight;
 
     *xp1 = x1;
     *yp1 = y1;
