@@ -71,6 +71,10 @@ static void hslider_draw_new(t_hslider *x, t_glist *glist)
              xpos - lmargin, ypos,
              xpos - lmargin + iow, ypos - IEMGUI_ZOOM(x) + ioh,
              x, 0);
+    sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxKNOB\n",
+             canvas, r, ypos + IEMGUI_ZOOM(x),
+             r, ypos + x->x_gui.x_h - IEMGUI_ZOOM(x),
+             1 + 2 * IEMGUI_ZOOM(x), x->x_gui.x_fcol, x);
     sys_vgui(".x%lx.c create text %d %d -text {%s} -anchor w \
              -font {{%s} -%d %s} -fill #%06x -tags [list %lxLABEL label text]\n",
              canvas, xpos + x->x_gui.x_ldx * IEMGUI_ZOOM(x),
@@ -78,10 +82,6 @@ static void hslider_draw_new(t_hslider *x, t_glist *glist)
              (strcmp(x->x_gui.x_lab->s_name, "empty") ? x->x_gui.x_lab->s_name : ""),
              x->x_gui.x_font, x->x_gui.x_fontsize * IEMGUI_ZOOM(x), sys_fontweight,
              x->x_gui.x_lcol, x);
-    sys_vgui(".x%lx.c create line %d %d %d %d -width %d -fill #%06x -tags %lxKNOB\n",
-             canvas, r, ypos + IEMGUI_ZOOM(x),
-             r, ypos + x->x_gui.x_h - IEMGUI_ZOOM(x),
-             1 + 2 * IEMGUI_ZOOM(x), x->x_gui.x_fcol, x);
 }
 
 static void hslider_draw_move(t_hslider *x, t_glist *glist)
@@ -107,12 +107,12 @@ static void hslider_draw_move(t_hslider *x, t_glist *glist)
              canvas, x, 0,
              xpos - lmargin, ypos,
              xpos - lmargin + iow, ypos - IEMGUI_ZOOM(x) + ioh);
-    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
-             canvas, x, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
-             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x));
     sys_vgui(".x%lx.c coords %lxKNOB %d %d %d %d\n",
              canvas, x, r, ypos + IEMGUI_ZOOM(x),
              r, ypos + x->x_gui.x_h - IEMGUI_ZOOM(x));
+    sys_vgui(".x%lx.c coords %lxLABEL %d %d\n",
+             canvas, x, xpos+x->x_gui.x_ldx * IEMGUI_ZOOM(x),
+             ypos+x->x_gui.x_ldy * IEMGUI_ZOOM(x));
 }
 
 static void hslider_draw_erase(t_hslider* x, t_glist* glist)
@@ -155,8 +155,9 @@ static void hslider_draw_io(t_hslider* x, t_glist* glist, int old_snd_rcv_flags)
              xpos - lmargin, ypos + x->x_gui.x_h + IEMGUI_ZOOM(x) - ioh,
              xpos - lmargin + iow, ypos + x->x_gui.x_h,
              x, 0);
-        /* keep knob above outlet */
+        /* keep these above outlet */
         sys_vgui(".x%lx.c raise %lxKNOB %lxOUT%d\n", canvas, x, x, 0);
+        sys_vgui(".x%lx.c raise %lxLABEL %lxKNOB\n", canvas, x, x);
     }
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_SND_FLAG) && x->x_gui.x_fsf.x_snd_able)
         sys_vgui(".x%lx.c delete %lxOUT%d\n", canvas, x, 0);
@@ -167,8 +168,9 @@ static void hslider_draw_io(t_hslider* x, t_glist* glist, int old_snd_rcv_flags)
              xpos - lmargin, ypos,
              xpos - lmargin + iow, ypos - IEMGUI_ZOOM(x) + ioh,
              x, 0);
-        /* keep label and knob above inlet */
+        /* keep these above inlet */
         sys_vgui(".x%lx.c raise %lxKNOB %lxIN%d\n", canvas, x, x, 0);
+        sys_vgui(".x%lx.c raise %lxLABEL %lxKNOB\n", canvas, x, x);
     }
     if(!(old_snd_rcv_flags & IEM_GUI_OLD_RCV_FLAG) && x->x_gui.x_fsf.x_rcv_able)
         sys_vgui(".x%lx.c delete %lxIN%d\n", canvas, x, 0);
@@ -543,10 +545,9 @@ static void *hslider_new(t_symbol *s, int argc, t_atom *argv)
 {
     t_hslider *x = (t_hslider *)pd_new(hslider_class);
     int w = IEM_SL_DEFAULTSIZE, h = IEM_GUI_DEFAULTSIZE;
-    int lilo = 0, ldx = -2, ldy = -8, f = 0, steady = 1;
+    int lilo = 0, ldx = -2, ldy = -8, steady = 1;
     int fs = 10;
     double min = 0.0, max = (double)(IEM_SL_DEFAULTSIZE-1);
-    char str[144];
     float v = 0;
 
     iem_inttosymargs(&x->x_gui.x_isa, 0);
