@@ -142,7 +142,7 @@ void sys_save_audio_params(
 void oss_init(void);
 #endif
 
-static void audio_init( void)
+static void audio_init(void)
 {
     static int initted = 0;
     if (initted)
@@ -417,7 +417,7 @@ void sys_close_audio(void)
 }
 
     /* open audio using whatever parameters were last used */
-void sys_reopen_audio( void)
+void sys_reopen_audio(void)
 {
     int naudioindev, audioindev[MAXAUDIOINDEV], chindev[MAXAUDIOINDEV];
     int naudiooutdev, audiooutdev[MAXAUDIOOUTDEV], choutdev[MAXAUDIOOUTDEV];
@@ -620,7 +620,7 @@ void sys_reportidle(void)
 
 /* this could later be set by a preference but for now it seems OK to just
 keep jack audio open but close unused audio devices for any other API */
-int audio_shouldkeepopen( void)
+int audio_shouldkeepopen(void)
 {
     return (sys_audioapi == API_JACK);
 }
@@ -709,7 +709,7 @@ static void audio_getdevs(char *indevlist, int *nindevs,
 }
 
 
-static void sys_listaudiodevs(void )
+static void sys_listaudiodevs(void)
 {
     char indevlist[MAXNDEV*DEVDESCSIZE], outdevlist[MAXNDEV*DEVDESCSIZE];
     int nindevs = 0, noutdevs = 0, i, canmulti = 0, cancallback = 0;
@@ -881,7 +881,7 @@ void sys_set_audio_settings_reopen(int naudioindev, int *audioindev, int nchinde
     else sched_reopenmeplease();
 }
 
-void sys_listdevs(void )
+void sys_listdevs(void)
 {
 #ifdef USEAPI_PORTAUDIO
     if (sys_audioapi == API_PORTAUDIO)
@@ -1069,7 +1069,7 @@ void sys_get_audio_apis(char *buf)
 #ifdef USEAPI_ALSA
 void alsa_putzeros(int n);
 void alsa_getzeros(int n);
-void alsa_printstate( void);
+void alsa_printstate(void);
 #endif
 
 /* convert a device name to a (1-based) device number.  (Output device if
@@ -1085,6 +1085,11 @@ int sys_audiodevnametonumber(int output, const char *name)
 
     if (output)
     {
+            /* try first for exact match */
+        for (i = 0; i < noutdevs; i++)
+            if (!strcmp(name, outdevlist + i * DEVDESCSIZE))
+                return (i);
+            /* failing that, a match up to end of shorter string */
         for (i = 0; i < noutdevs; i++)
         {
             unsigned long comp = strlen(name);
@@ -1096,6 +1101,9 @@ int sys_audiodevnametonumber(int output, const char *name)
     }
     else
     {
+        for (i = 0; i < nindevs; i++)
+            if (!strcmp(name, indevlist + i * DEVDESCSIZE))
+                return (i);
         for (i = 0; i < nindevs; i++)
         {
             unsigned long comp = strlen(name);
