@@ -457,7 +457,7 @@ static void pdcontrol_args(t_pdcontrol *x, t_floatarg f)
 static void pdcontrol_browse(t_pdcontrol *x, t_symbol *s)
 {
     char buf[MAXPDSTRING];
-    snprintf(buf, MAXPDSTRING, "::pd_menucommands::menu_openfile %s\n",
+    snprintf(buf, MAXPDSTRING, "::pd_menucommands::menu_openfile {%s}\n",
         s->s_name);
     buf[MAXPDSTRING-1] = 0;
     sys_gui(buf);
