@@ -482,7 +482,7 @@ typedef struct _oscformat
 {
     t_object x_obj;
     char *x_pathbuf;
-    int x_pathsize;
+    size_t x_pathsize;
     t_symbol *x_format;
 } t_oscformat;
 
@@ -490,7 +490,7 @@ static void oscformat_set(t_oscformat *x, t_symbol *s, int argc, t_atom *argv)
 {
     char buf[MAXPDSTRING];
     int i;
-    unsigned long newsize;
+    size_t newsize;
     *x->x_pathbuf = 0;
     buf[0] = '/';
     for (i = 0; i < argc; i++)
@@ -501,7 +501,7 @@ static void oscformat_set(t_oscformat *x, t_symbol *s, int argc, t_atom *argv)
         if ((newsize = strlen(buf) + strlen(x->x_pathbuf) + 1) > x->x_pathsize)
         {
             x->x_pathbuf = resizebytes(x->x_pathbuf, x->x_pathsize, newsize);
-            x->x_pathsize = (int)newsize;
+            x->x_pathsize = newsize;
         }
         strcat(x->x_pathbuf, buf);
     }
@@ -509,7 +509,7 @@ static void oscformat_set(t_oscformat *x, t_symbol *s, int argc, t_atom *argv)
 
 static void oscformat_format(t_oscformat *x, t_symbol *s)
 {
-    char *sp;
+    const char *sp;
     for (sp = s->s_name; *sp; sp++)
     {
         if (*sp != 'f' && *sp != 'i' && *sp != 's' && *sp != 'b')
@@ -548,7 +548,8 @@ static void oscformat_list(t_oscformat *x, t_symbol *s, int argc, t_atom *argv)
 {
     int typeindex = 0, j, msgindex, msgsize, datastart, ndata;
     t_atom *msg;
-    char *sp, *formatp = x->x_format->s_name, typecode;
+    const char *sp, *formatp = x->x_format->s_name;
+    char typecode;
         /* pass 1: go through args to find overall message size */
     for (j = ndata = 0, sp = formatp, msgindex = 0; j < argc;)
     {
@@ -558,7 +559,15 @@ static void oscformat_list(t_oscformat *x, t_symbol *s, int argc, t_atom *argv)
             typecode = 's';
         else typecode = 'f';
         if (typecode == 's')
-            msgindex += ROUNDUPTO4(strlen(argv[j].a_w.w_symbol->s_name) + 1);
+        {
+            if (argv[j].a_type == A_SYMBOL)
+                msgindex += ROUNDUPTO4(strlen(argv[j].a_w.w_symbol->s_name) + 1);
+            else
+            {
+                pd_error(x, "oscformat: expected symbol for argument %d", j+1);
+                return;
+            }
+        }
         else if (typecode == 'b')
         {
             int blobsize = 0x7fffffff, blobindex;
