@@ -61,13 +61,13 @@ proc ::pd_menucommands::menu_print {mytoplevel} {
 # functions called from Edit menu
 
 proc ::pd_menucommands::menu_undo {} {
-    if {$::focused_window eq $::undo_toplevel && $::undo_action ne "no"} {
+    if { $::focused_window ne ".pdwindow" } {
         pdsend "$::focused_window undo"
     }
 }
 
 proc ::pd_menucommands::menu_redo {} {
-    if {$::focused_window eq $::undo_toplevel && $::redo_action ne "no"} {
+    if { $::focused_window ne ".pdwindow" } {
         pdsend "$::focused_window redo"
     }
 }
@@ -259,7 +259,7 @@ proc ::pd_menucommands::menu_doc_open {dir basename} {
         set fullpath [file normalize [file join $dirname $basename]]
         set dirname [file dirname $fullpath]
         set basename [file tail $fullpath]
-        pdsend "pd open [enquote_path $basename] [enquote_path $dirname]"
+        pdsend "pd open [enquote_path $basename] [enquote_path $dirname] 1"
     } else {
         ::pd_menucommands::menu_openfile "$dirname/$basename"
     }
